##Passing of attributes to the macro

Welcome to the **dynamic attributes** setting into the log event! Every log event can transport any number of additional attributes, which will participate in all subsequent operations associated with it.

How is it imlemented?

Actually, `BH_LOG` macro returns functional object, which accepts variadic pack. You can set this pack in four ways.

**Attribute::make way**

Explicitly create attributes objects using helper `attributes::make` function:

```
BH_LOG(log, level::debug, "debug event")(
    attribute::make("host", "localhost"),
    attribute::make("answer", 42.0),
    attribute::make("code", 404)
);
```

We can pass into the `attribute::make()` call one more parameter ( 'L' | 'E' | 'G' | 'T' | 'U' ) that will be treated as the type of the attribute.

  * 'L' - Local attribute. Attribute generated by user;
  * 'E' - Event attribute. Attribute, that doesn't specified by user, like message or timestamp;
  * 'G' - Global attribute. Logger's object specific attribute, which can be attached to the logger object and transported with it;
  * 'T' - Thread attribute. Thread-specific attributes, like thread id;
  * 'U' - Universe attribute.  Program global attributes, like process id.

For example

```
BH_LOG(log, level::debug, "debug event")(attribute::make("host", "localhost",'G'),
```

*All additional (in the second parenthesis) attributes have type `L` by default.*


**Keywords way** 

You can use registered **keywords**:
```
BH_LOG(log, level::debug, "debug event")(
    keyword::host() = "localhost",
    keyword::answer() = 42.0,
    keyword::code() = 404
);
```

Before using keywords you should register it by youself. Blackhole has three registered keyword by default `timestamp`, `severity` and `message`. Once registered keyword couldn't be re-registered. 

??????????????????How to register keyword???????????????????????????????

In fact it is syntactic sugar over the `attribute::make` method. Registered keywords have defined `operator=`, which returns attribute object therefore the first and the second method can be combined:

```
BH_LOG(log, level::debug, "debug event")(
    attribute::make("host", "localhost"),
    keyword::answer() = 42.0,
    attribute::make("code", 404)
);
```
??????????????
Keywords can be useful not only for these purposes. More details about them will be discussed in reference documentation.
??????????????


**Key-value way**

This method allows you to specify a list of attributes as key-value pairs, where the key should be converted string and value - should be implicitly converted to `attribute_value_t`. In code it looks like this:

```
BH_LOG(log, level::debug, "debug event")(
    "host", "localhost",
    "answer", 42.0,
    "code", 404
);
```

Argument correctness is verified at compile time, so if something goes wrong will be given a human-readable `static_assert`.

**Initialization lists way**

This looks like follows:

```
BH_LOG(log, level::debug, "debug event")(attribute::list({
    {"host", "localhost"},
    {"answer", 42.0},
    {"code", 404}
}));
```

Some compilers allows you not to specify `attribute::list` type explicitly. For example:

```
BH_LOG(log, level::debug, "debug event")({
    {"host", "localhost"},
    {"answer", 42.0},
    {"code", 404}
});
```

*Actually, only GCC 4.6 doesn't allow you to write like this, because it violates the C++11 Standard (see '-fno-deduce-init-list' extension for more details).*

Which of the following methods to choose - is your decision, depending on what you prefer.

[Back to contents](contents.md)